"""
'CHROMAT'
copyright (c) 2023 hex benjamin
full license available at /COPYING
"""

from .utility.console import CONSOLE
from .swatch import Swatch
from .palette import Palette

__all__ = [
    "CONSOLE",
    "utility",
    "Swatch",
    "Palette",
]

LAUNCH_STRING = "CHROMAT v0.0.6"
